@@ -33,6 +33,21 @@ def _nanmin(m, n):
     return k_1 if np.isnan(k_0) else k_0
 
 
+def _wrapped_qr(a):
+    """
+    A wrapper for np.linalg.qr that handles arrays with 0 rows
+
+    Notes: Created for tsqr so as to manage cases with uncertain
+    array dimensions. In particular, the case where arrays have
+    (uncertain) chunks with 0 rows.
+    """
+    # workaround may be removed when numpy stops rejecting edge cases
+    if a.shape[0] == 0:
+        return np.zeros((0, 0)), np.zeros((0, a.shape[1]))
+    else:
+        return np.linalg.qr(a)
+
+
 def tsqr(data, compute_svd=False, _max_vchunk_size=None):
     """ Direct Tall-and-Skinny QR algorithm
 
@@ -60,14 +75,16 @@ def tsqr(data, compute_svd=False, _max_vchunk_size=None):
     Notes
     -----
     With ``k`` blocks of size ``(m, n)``, this algorithm has memory use that
-    scales as ``m * n * k``.
+    scales as ``k * m * n``.
 
     The implementation here is the recursive variant due to the ultimate
-    need for one "single core" QR decomposition. Given m blocks, the
-    "single core" QR decomposition will have to work with a ``(k * n, n)``
-    matrix, and ``k * n`` may be substantially larger than ``m``.
+    need for one "single core" QR decomposition. In the non-recursive version
+    of the algorithm, given ``k`` blocks, after ``k`` ``m * n`` QR
+    decompositions, there will be a "single core" QR decomposition that will
+    have to work with a ``(k * n, n)`` matrix.
 
-    As such, (if ``m / n >= 2``) recursion will be applied as necessary
+    Here, recursion is applied as necessary to ensure that ``k * n`` is not
+    larger than ``m`` (if ``m / n >= 2``). In particular, this is done
     to ensure that single core computations do not have to work on blocks
     larger than ``(m, n)``.
 
@@ -108,7 +125,7 @@ def tsqr(data, compute_svd=False, _max_vchunk_size=None):
 
     # Block qr
     name_qr_st1 = 'qr' + token
-    dsk_qr_st1 = top(np.linalg.qr, name_qr_st1, 'ij', data.name, 'ij',
+    dsk_qr_st1 = top(_wrapped_qr, name_qr_st1, 'ij', data.name, 'ij',
                      numblocks={data.name: numblocks})
     dsk.update_with_key(dsk_qr_st1, key=name_qr_st1)
 
