@@ -52,26 +52,32 @@ def test_arr_like(funcname, shape, dtype, chunks):
         assert (np_r == np.asarray(da_r)).all()
 
 
-def test_linspace():
-    darr = da.linspace(6, 49, chunks=5)
-    nparr = np.linspace(6, 49)
+@pytest.mark.parametrize("endpoint", [True, False])
+def test_linspace(endpoint):
+    darr = da.linspace(6, 49, endpoint=endpoint, chunks=5)
+    nparr = np.linspace(6, 49, endpoint=endpoint)
     assert_eq(darr, nparr)
 
-    darr = da.linspace(1.4, 4.9, chunks=5, num=13)
-    nparr = np.linspace(1.4, 4.9, num=13)
+    darr = da.linspace(1.4, 4.9, endpoint=endpoint, chunks=5, num=13)
+    nparr = np.linspace(1.4, 4.9, endpoint=endpoint, num=13)
     assert_eq(darr, nparr)
 
-    darr = da.linspace(6, 49, chunks=5, dtype=float)
-    nparr = np.linspace(6, 49, dtype=float)
+    darr = da.linspace(6, 49, endpoint=endpoint, chunks=5, dtype=float)
+    nparr = np.linspace(6, 49, endpoint=endpoint, dtype=float)
     assert_eq(darr, nparr)
 
-    darr = da.linspace(1.4, 4.9, chunks=5, num=13, dtype=int)
-    nparr = np.linspace(1.4, 4.9, num=13, dtype=int)
+    darr, dstep = da.linspace(6, 49, endpoint=endpoint, chunks=5, retstep=True)
+    nparr, npstep = np.linspace(6, 49, endpoint=endpoint, retstep=True)
+    assert np.allclose(dstep, npstep)
     assert_eq(darr, nparr)
-    assert (sorted(da.linspace(1.4, 4.9, chunks=5, num=13).dask) ==
-            sorted(da.linspace(1.4, 4.9, chunks=5, num=13).dask))
-    assert (sorted(da.linspace(6, 49, chunks=5, dtype=float).dask) ==
-            sorted(da.linspace(6, 49, chunks=5, dtype=float).dask))
+
+    darr = da.linspace(1.4, 4.9, endpoint=endpoint, chunks=5, num=13, dtype=int)
+    nparr = np.linspace(1.4, 4.9, num=13, endpoint=endpoint, dtype=int)
+    assert_eq(darr, nparr)
+    assert (sorted(da.linspace(1.4, 4.9, endpoint=endpoint, chunks=5, num=13).dask) ==
+            sorted(da.linspace(1.4, 4.9, endpoint=endpoint, chunks=5, num=13).dask))
+    assert (sorted(da.linspace(6, 49, endpoint=endpoint, chunks=5, dtype=float).dask) ==
+            sorted(da.linspace(6, 49, endpoint=endpoint, chunks=5, dtype=float).dask))
 
 
 def test_arange():
@@ -113,9 +119,37 @@ def test_arange():
     nparr = np.arange(0, -1, 0.5)
     assert_eq(darr, nparr)
 
+    # Unexpected or missing kwargs
+    with pytest.raises(TypeError) as exc:
+        da.arange(10, chunks=-1, whatsthis=1)
+    assert 'whatsthis' in str(exc)
+    with pytest.raises(TypeError) as exc:
+        da.arange(10)
+    assert 'chunks' in str(exc)
 
-def test_arange_has_dtype():
-    assert da.arange(5, chunks=2).dtype == np.arange(5).dtype
+
+@pytest.mark.parametrize("start,stop,step,dtype", [
+    (0, 1, 1, None),  # int64
+    (1.5, 2, 1, None),  # float64
+    (1, 2.5, 1, None),  # float64
+    (1, 2, .5, None),  # float64
+    (np.float32(1), np.float32(2), np.float32(1), None),  # promoted to float64
+    (np.int32(1), np.int32(2), np.int32(1), None),  # promoted to int64
+    (np.uint32(1), np.uint32(2), np.uint32(1), None),  # promoted to int64
+    (np.uint64(1), np.uint64(2), np.uint64(1), None),  # promoted to float64
+    (np.uint32(1), np.uint32(2), np.uint32(1), np.uint32),
+    (np.uint64(1), np.uint64(2), np.uint64(1), np.uint64),
+    # numpy.arange gives unexpected results
+    # https://github.com/numpy/numpy/issues/11505
+    # (1j, 2, 1, None),
+    # (1, 2j, 1, None),
+    # (1, 2, 1j, None),
+    # (1+2j, 2+3j, 1+.1j, None),
+])
+def test_arange_dtypes(start, stop, step, dtype):
+    a_np = np.arange(start, stop, step, dtype=dtype)
+    a_da = da.arange(start, stop, step, dtype=dtype, chunks=-1)
+    assert_eq(a_np, a_da)
 
 
 @pytest.mark.xfail(reason="Casting floats to ints is not supported since edge"
@@ -416,3 +450,58 @@ def test_tile_array_reps(shape, chunks, reps):
 
     with pytest.raises(NotImplementedError):
         da.tile(d, reps)
+
+
+@pytest.mark.parametrize('shape, chunks, pad_width, mode, kwargs', [
+    ((10,), (3,), 1, 'constant', {}),
+    ((10,), (3,), 2, 'constant', {'constant_values': -1}),
+    ((10,), (3,), ((2, 3)), 'constant', {'constant_values': (-1, -2)}),
+    (
+        (10, 11), (4, 5), ((1, 4), (2, 3)), 'constant',
+        {'constant_values': ((-1, -2), (2, 1))}
+    ),
+    ((10,), (3,), 3, 'edge', {}),
+    ((10,), (3,), 3, 'linear_ramp', {}),
+    ((10,), (3,), 3, 'linear_ramp', {'end_values': 0}),
+    ((10, 11), (4, 5), ((1, 4), (2, 3)), 'reflect', {}),
+    ((10, 11), (4, 5), ((1, 4), (2, 3)), 'symmetric', {}),
+    ((10, 11), (4, 5), ((1, 4), (2, 3)), 'wrap', {}),
+    ((10,), (3,), ((2, 3)), 'maximum', {'stat_length': (1, 2)}),
+    (
+        (10, 11), (4, 5), ((1, 4), (2, 3)), 'mean',
+        {'stat_length': ((3, 4), (2, 1))}
+    ),
+    ((10,), (3,), ((2, 3)), 'minimum', {'stat_length': (2, 3)}),
+])
+def test_pad(shape, chunks, pad_width, mode, kwargs):
+    np_a = np.random.random(shape)
+    da_a = da.from_array(np_a, chunks=chunks)
+
+    np_r = np.pad(np_a, pad_width, mode, **kwargs)
+    da_r = da.pad(da_a, pad_width, mode, **kwargs)
+
+    assert_eq(np_r, da_r)
+
+
+@pytest.mark.parametrize('kwargs', [
+    {},
+    {"scaler": 2},
+])
+def test_pad_udf(kwargs):
+    def udf_pad(vector, pad_width, iaxis, kwargs):
+        scaler = kwargs.get("scaler", 1)
+        vector[:pad_width[0]] = -scaler * pad_width[0]
+        vector[-pad_width[1]:] = scaler * pad_width[1]
+        return vector
+
+    shape = (10, 11)
+    chunks = (4, 5)
+    pad_width = ((1, 2), (2, 3))
+
+    np_a = np.random.random(shape)
+    da_a = da.from_array(np_a, chunks=chunks)
+
+    np_r = np.pad(np_a, pad_width, udf_pad, kwargs=kwargs)
+    da_r = da.pad(da_a, pad_width, udf_pad, kwargs=kwargs)
+
+    assert_eq(np_r, da_r)
