@@ -268,7 +268,7 @@ def test_turn_off_fusion():
 
     a = y.__dask_optimize__(y.dask, y.__dask_keys__())
 
-    with dask.set_options(fuse_ave_width=0):
+    with dask.config.set(fuse_ave_width=0):
         b = y.__dask_optimize__(y.dask, y.__dask_keys__())
 
     assert dask.get(a, y.__dask_keys__()) == dask.get(b, y.__dask_keys__())
