@@ -420,14 +420,14 @@ def test_array_cumreduction_out(func):
     (np.sort, da.topk),
     (np.argsort, da.argtopk),
 ])
-@pytest.mark.parametrize('split_every', [None, 2])
+@pytest.mark.parametrize('split_every', [None, 2, 4, 8])
 def test_topk_argtopk1(npfunc, daskfunc, split_every):
     # Test data
     k = 5
-    a = da.random.random(1000, chunks=250)
+    # Test at least 3 levels of aggregation when split_every=2
+    # to stress the different chunk, combine, aggregate kernels
+    a = da.random.random(800, chunks=((120, 80, 100, 200, 300), ))
     b = da.random.random((10, 20, 30), chunks=(4, 8, 8))
-    c = da.from_array(np.array([(1, 'Hello'), (2, 'World')], dtype=[('foo', int), ('bar', '<U5')]),
-                      chunks=1)
 
     # 1-dimensional arrays
     # top 5 elements, sorted descending
@@ -459,12 +459,6 @@ def test_topk_argtopk1(npfunc, daskfunc, split_every):
     with pytest.raises(ValueError):
         daskfunc(b, -k, axis=3, split_every=split_every)
 
-    # structured arrays
-    assert_eq(npfunc(c, axis=0)[-1:][::-1],
-              daskfunc(c, 1, split_every=split_every))
-    assert_eq(npfunc(c, axis=0)[:1],
-              daskfunc(c, -1, split_every=split_every))
-
 
 def test_topk_argtopk2():
     a = da.random.random((10, 20, 30), chunks=(4, 8, 8))
