@@ -1,7 +1,6 @@
 from __future__ import print_function, division, absolute_import
 
 import ast
-import copy
 import os
 import sys
 import threading
@@ -10,6 +9,8 @@ try:
 except ImportError:
     yaml = None
 
+from .compatibility import makedirs
+
 
 no_default = '__no_default__'
 
@@ -22,7 +23,10 @@ paths = [
 ]
 
 if 'DASK_CONFIG' in os.environ:
-    paths.append(os.environ['DASK_CONFIG'])
+    PATH = os.environ['DASK_CONFIG']
+    paths.append(PATH)
+else:
+    PATH = os.path.join(os.path.expanduser('~'), '.config', 'dask')
 
 
 global_config = config = {}
@@ -159,7 +163,7 @@ def collect_env(env=None):
 
 def ensure_file(
         source,
-        destination=os.path.join(os.path.expanduser('~'), '.config', 'dask'),
+        destination=None,
         comment=True):
     """
     Copy file to default location if it does not already exist
@@ -173,41 +177,52 @@ def ensure_file(
 
     Parameters
     ----------
-    source: string, filename
-        source configuration file, typically within a source directory
-    destination: string, filename
-        destination filename, typically ~/.config/dask
-    comment: bool, True by default
-        Whether or not to comment out the config file when copying
+    source : string, filename
+        Source configuration file, typically within a source directory.
+    destination : string, directory
+        Destination directory. Configurable by ``DASK_CONFIG`` environment
+        variable, falling back to ~/.config/dask.
+    comment : bool, True by default
+        Whether or not to comment out the config file when copying.
     """
-    if not os.path.splitext(destination)[1].strip('.'):
-        _, filename = os.path.split(source)
-        destination = os.path.join(destination, filename)
+    if destination is None:
+        destination = PATH
 
-    if not os.path.exists(os.path.dirname(destination)):
-        os.makedirs(os.path.dirname(destination))
+    # destination is a file and already exists, never overwrite
+    if os.path.isfile(destination):
+        return
 
-    if not os.path.exists(destination):
-        # Atomically create destination.  Parallel testing discovered
-        # a race condition where a process can be busy creating the
-        # destination while another process reads an empty config file.
-        tmp = '%s.tmp.%d' % (destination, os.getpid())
-        with open(source) as f:
-            lines = list(f)
+    # If destination is not an existing file, interpret as a directory,
+    # use the source basename as the filename
+    directory = destination
+    destination = os.path.join(directory, os.path.basename(source))
 
-        if comment:
-            lines = ['# ' + line
-                     if line.strip() and not line.startswith('#')
-                     else line
-                     for line in lines]
+    try:
+        if not os.path.exists(destination):
+            makedirs(directory, exist_ok=True)
 
-        with open(tmp, 'w') as f:
-            f.write(''.join(lines))
+            # Atomically create destination.  Parallel testing discovered
+            # a race condition where a process can be busy creating the
+            # destination while another process reads an empty config file.
+            tmp = '%s.tmp.%d' % (destination, os.getpid())
+            with open(source) as f:
+                lines = list(f)
 
-        try:
-            os.rename(tmp, destination)
-        except OSError:
-            os.remove(tmp)
+            if comment:
+                lines = ['# ' + line
+                         if line.strip() and not line.startswith('#')
+                         else line
+                         for line in lines]
+
+            with open(tmp, 'w') as f:
+                f.write(''.join(lines))
+
+            try:
+                os.rename(tmp, destination)
+            except OSError:
+                os.remove(tmp)
+    except OSError:
+        pass
 
 
 class set(object):
@@ -229,26 +244,63 @@ class set(object):
 
         with lock:
             self.config = config
-            self.old = copy.deepcopy(config)
-
-            def assign(keys, value, d):
-                key = keys[0]
-                if len(keys) == 1:
-                    d[keys[0]] = value
-                else:
-                    if key not in d:
-                        d[key] = {}
-                    assign(keys[1:], value, d[key])
+            self.old = {}
 
             for key, value in kwargs.items():
-                assign(key.split('.'), value, config)
+                self._assign(key.split('.'), value, config, old=self.old)
 
     def __enter__(self):
-        return config
+        return self.config
 
     def __exit__(self, type, value, traceback):
-        self.config.clear()
-        self.config.update(self.old)
+        for keys, value in self.old.items():
+            if value == '--delete--':
+                d = self.config
+                try:
+                    while len(keys) > 1:
+                        d = d[keys[0]]
+                        keys = keys[1:]
+                    del d[keys[0]]
+                except KeyError:
+                    pass
+            else:
+                self._assign(keys, value, self.config)
+
+    @classmethod
+    def _assign(cls, keys, value, d, old=None, path=[]):
+        """ Assign value into a nested configuration dictionary
+
+        Optionally record the old values in old
+
+        Parameters
+        ----------
+        keys: Sequence[str]
+            The nested path of keys to assign the value, similar to toolz.put_in
+        value: object
+        d: dict
+            The part of the nested dictionary into which we want to assign the
+            value
+        old: dict, optional
+            If provided this will hold the old values
+        path: List[str]
+            Used internally to hold the path of old values
+        """
+        if len(keys) == 1:
+            if old is not None:
+                path_key = tuple(path + [keys[0]])
+                if keys[0] in d:
+                    old[path_key] = d[keys[0]]
+                else:
+                    old[path_key] = '--delete--'
+            d[keys[0]] = value
+        else:
+            key = keys[0]
+            if key not in d:
+                d[key] = {}
+                if old is not None:
+                    old[tuple(path + [key])] = '--delete--'
+                old = None
+            cls._assign(keys[1:], value, d[key], path=path + [key], old=old)
 
 
 def collect(paths=paths, env=None):
