@@ -505,6 +505,11 @@ def _read_pyarrow(fs, fs_token, paths, columns=None, filters=None,
         columns = list(columns)
 
     dataset = pq.ParquetDataset(paths, filesystem=get_pyarrow_filesystem(fs))
+    if dataset.partitions is not None:
+        partitions = [n for n in dataset.partitions.partition_names
+                      if n is not None]
+    else:
+        partitions = []
     schema = dataset.schema.to_arrow_schema()
     has_pandas_metadata = schema.metadata is not None and b'pandas' in schema.metadata
 
@@ -528,6 +533,7 @@ def _read_pyarrow(fs, fs_token, paths, columns=None, filters=None,
             index_names = [name_storage_mapping.get(name, name)
                            for name in index_names]
 
+    column_names += [p for p in partitions if p not in column_names]
     column_names, index_names, out_type = _normalize_index_columns(
         columns, column_names, index, index_names)
 
@@ -764,7 +770,7 @@ def _read_pyarrow_parquet_piece(fs, piece, columns, index_cols, is_series,
     if is_series:
         return df[df.columns[0]]
     else:
-        return df
+        return df[columns]
 
 
 _pyarrow_write_table_kwargs = {'row_group_size', 'version', 'use_dictionary',
@@ -893,12 +899,14 @@ def read_parquet(path, columns=None, filters=None, categories=None, index=None,
 
     Parameters
     ----------
-    path : string or list
+    path : string, list or fastparquet.ParquetFile
         Source directory for data, or path(s) to individual parquet files.
         Prefix with a protocol like ``s3://`` to read from alternative
         filesystems. To read from multiple files you can pass a globstring or a
         list of paths, with the caveat that they must all have the same
         protocol.
+        Alternatively, also accepts a previously opened
+        fastparquet.ParquetFile()
     columns : string, list or None (default)
         Field name(s) to read in as columns in the output. By default all
         non-index fields will be read (as determined by the pandas parquet
