@@ -13,9 +13,8 @@ from .core import DataFrame, Series, _Frame, _concat, map_partitions
 from .hashing import hash_pandas_object
 from .utils import PANDAS_VERSION
 
-from .. import base
+from .. import base, config
 from ..base import tokenize, compute, compute_as_if_collection
-from ..context import _globals
 from ..delayed import delayed
 from ..sizeof import sizeof
 from ..utils import digit, insert, M
@@ -229,7 +228,7 @@ def rearrange_by_divisions(df, column, divisions, max_branch=None, shuffle=None)
 
 def rearrange_by_column(df, col, npartitions=None, max_branch=None,
                         shuffle=None, compute=None):
-    shuffle = shuffle or _globals.get('shuffle', 'disk')
+    shuffle = shuffle or config.get('shuffle', 'disk')
     if shuffle == 'disk':
         return rearrange_by_column_disk(df, col, npartitions, compute=compute)
     elif shuffle == 'tasks':
@@ -242,7 +241,7 @@ class maybe_buffered_partd(object):
     """If serialized, will return non-buffered partd. Otherwise returns a
     buffered partd"""
     def __init__(self, buffer=True, tempdir=None):
-        self.tempdir = tempdir or _globals.get('temporary_directory')
+        self.tempdir = tempdir or config.get('temporary_directory', None)
         self.buffer = buffer
 
     def __reduce__(self):
