@@ -2,6 +2,7 @@ from __future__ import absolute_import, division, print_function
 
 from collections import Iterator
 import operator
+import types
 import uuid
 
 try:
@@ -434,6 +435,11 @@ class Delayed(DaskMethodsMixin, OperatorMethodMixin):
 
     __nonzero__ = __bool__
 
+    def __get__(self, instance, cls):
+        if instance is None:
+            return self
+        return types.MethodType(self, instance)
+
     @classmethod
     def _get_binary_operator(cls, op, inv=False):
         method = delayed(right(op) if inv else op, pure=True)
@@ -520,6 +526,12 @@ for op in [operator.abs, operator.neg, operator.pos, operator.invert,
     Delayed._bind_operator(op)
 
 
+try:
+    Delayed._bind_operator(operator.matmul)
+except AttributeError:
+    pass
+
+
 def single_key(seq):
     """ Pick out the only element of this list, a list of keys """
     return seq[0]
