@@ -2,6 +2,7 @@ from collections import namedtuple
 from operator import add, setitem
 import pickle
 from random import random
+import types
 
 from toolz import identity, partial, merge
 import pytest
@@ -12,6 +13,11 @@ from dask.compatibility import PY2, PY3
 from dask.delayed import delayed, to_task_dask, Delayed
 from dask.utils_test import inc
 
+try:
+    from operator import matmul
+except ImportError:
+    matmul = None
+
 
 class Tuple(object):
     __dask_scheduler__ = staticmethod(dask.threaded.get)
@@ -102,6 +108,15 @@ def test_operators():
     assert (a > 2).compute()
     assert (a ** 2).compute() == 100
 
+    if matmul:
+        class dummy:
+            def __matmul__(self, other):
+                return 4
+        c = delayed(dummy())  # noqa
+        d = delayed(dummy())  # noqa
+
+        assert (eval('c @ d')).compute() == 4
+
 
 def test_methods():
     a = delayed("a b c d e")
@@ -502,3 +517,47 @@ def test_keys_from_array():
     xs = [delayed(inc)(x) for x in X]
 
     _check_dsk(xs[0].dask)
+
+
+# Mostly copied from https://github.com/pytoolz/toolz/pull/220
+def test_delayed_decorator_on_method():
+    class A(object):
+        BASE = 10
+
+        def __init__(self, base):
+            self.BASE = base
+
+        @delayed
+        def addmethod(self, x, y):
+            return self.BASE + x + y
+
+        @classmethod
+        @delayed
+        def addclass(cls, x, y):
+            return cls.BASE + x + y
+
+        @staticmethod
+        @delayed
+        def addstatic(x, y):
+            return x + y
+
+    a = A(100)
+    assert a.addmethod(3, 4).compute() == 107
+    assert A.addmethod(a, 3, 4).compute() == 107
+
+    assert a.addclass(3, 4).compute() == 17
+    assert A.addclass(3, 4).compute() == 17
+
+    assert a.addstatic(3, 4).compute() == 7
+    assert A.addstatic(3, 4).compute() == 7
+
+    # We want the decorated methods to be actual methods for instance methods
+    # and class methods since their first arguments are the object and the
+    # class respectively. Or in other words, the first argument is generated by
+    # the runtime based on the object/class before the dot.
+    assert isinstance(a.addmethod, types.MethodType)
+    assert isinstance(A.addclass, types.MethodType)
+
+    # For static methods (and regular functions), the decorated methods should
+    # be Delayed objects.
+    assert isinstance(A.addstatic, Delayed)
